// Load the AWS SDK for Node.js
var AWS = require('aws-sdk');
var exec = require("child_process").execSync;
var concat = require("concat-stream");
var aesgcm = require("aes-gcm-stream")
/**
 * Don't hard-code your credentials!
 * Export the following environment variables instead:
 *
 * export AWS_ACCESS_KEY_ID='AKID'
 * export AWS_SECRET_ACCESS_KEY='SECRET'
 * export AWS_S3_REGION='us-west-2'
 */

// Set your region for future requests.
//AWS.config.region = 'us-west-2';
//process.env.AWS_PROFILE
AWS.config.region = "us-west-2";

var bucket="firmware-npm";

var fs = require('fs');
var streamBuffers = require("stream-buffers")
var yauzl = require("yauzl");
var NO_DECRYPT = false;

const crypto = require('crypto');
const path = require('path');
const index_name = 'dfu_index.json';

function encrypt_dfu (file_path, key) {

    const iv = crypto.randomBytes(12);
    var file = fs.readFileSync(file_path);
    basename = path.posix.basename(file_path);
    var e = gcm.encrypt(key, iv, file, new Buffer(basename));
    e.ciphertext = e.ciphertext.toString('Base64');
    e.auth_tag = e.auth_tag.toString('Base64');
    e.iv = iv.toString('Base64');
    return e;
}


function get_index (callback) {
    var s3 = new AWS.S3({params: {Bucket: bucket, Key: index_name}});
    var i = s3.getObject(function(err, data) {
        if (err) {
            console.log(err, err.stack); // an error occurred
            callback(err);
        }
        else{
            index = JSON.parse(data.Body.toString());
            callback (null,index);
        }
    });
}

function put_index (index) {
    var s3obj = new AWS.S3({params: {Bucket: bucket, Key: index_name}});
    s3obj.upload({Body: index}).
        //on('httpUploadProgress', function(evt) { console.log(evt); }).
        send(function(err, data) {
            //console.log(err, data);
        });
}

function update_index (file_name) {
    get_index (function(err,index) {
        var entry = {};
        entry.name = file_name;
        entry.time = new Date().toISOString();
        index.history.push (entry);
        index.latest = file_name;
        put_index (JSON.stringify(index));
    });
}

function put_dfu(dfu_file_name) {
    const key = crypto.randomBytes(16);
    console.log ("This is the key for this release: %s", key.toString('base64'));
    const encrypted_dfu = encrypt_dfu (dfu_file_name, key);

    const s3name = path.posix.basename(dfu_file_name);
    var s3obj = new AWS.S3({params: {Bucket: bucket, Key: s3name}});
    s3obj.upload({Body: JSON.stringify(encrypted_dfu)}).
        //on('httpUploadProgress', function(evt) { console.log(evt); }).
        send(function(err, data) {
            //console.log(err, data);
            update_index (s3name);
        });
}

function get_dfu(b64key, opts, vers) {
  var B = opts.bucket || bucket;
  return new Promise((resolve, reject) => {
    function cb(err, res){
      if (err) return reject(err);
      console.log("resolving",res);
      resolve(res);
    }
    console.log("VERSION",vers);
    var getVersionFromIndex = (latest) => latest.split("-")[1].split(".z")[0] + "-release" // tap-v0.12.3.zip --> c0.12.3-release

    var getlocal = (relative) => {
      var fullpath = path.join(process.cwd(), relative);
      var pathparts = fullpath.split(path.sep);
      var version;
      while (pathparts.length){
        var section = pathparts.pop();
        if (section.indexOf('tap-v') === 0){
          version = section.split("-v")[1].split(" ")[0] + "-release"
          break;
        }
      }
      if (!version)
      {
        var dirpath = fullpath.split(path.sep);
        dirpath.pop();
        var cwd = dirpath.join(path.sep);
        var v1 = exec("git describe --abbrev=4 --dirty --always --tags", {cwd : cwd}).toString().trim();
        var v2 =  exec("git symbolic-ref --short -q HEAD", {cwd : cwd}).toString().trim()
        version =  v1 + "-" + v2
      }
      return ({
        binary : fs.readFileSync(fullpath),
        version : version
      })
    }

    console.log("VERSION",vers)

    if (!vers){
      var entries = {
        patchos : opts.patchos ? getlocal(opts.patchos) : null,
        tapos : opts.tapos ? getlocal(opts.tapos) : null,
        foible_application : opts.foible_application ? getlocal(opts.foible_application) : null
      }

      //all local, no s3
      if (Object.keys(entries).every(j => entries[j] ? true : false))
        return cb(null, entries);
    } else {
      var entries = {
        patchos : null,
        tapos : null,
        foible_application : null
      }
    }

    if (NO_DECRYPT) return cb(new Error("unable to load node-aes-gcm"))

    get_index(function(err, index){
      if (err) return cb(err);
      var s3 = new AWS.S3();
      var params = {Bucket: bucket, Key: vers || index.latest};
      console.log(index.latest, b64key);
      var blob = new Buffer(0);
      var decrypt = aesgcm.decrypt({key : b64key})
      var rs = s3.getObject(params)
        .createReadStream();
      rs.on('error', (e) => {
        console.log("AWS ERROR")
      });

      rs.pipe(decrypt)
        .pipe(concat((plaintext) => {
          yauzl.fromBuffer(plaintext,function(err,res){
            if (err) return cb(err);

            res.on('entry',(entry) => {
              //console.log(entry);
              var fileName = entry.fileName;
              var type = entry.type; // 'Directory' or 'File'
              var size = entry.size;
              console.log("entry", fileName)
              Object.keys(entries).forEach(k => {
                if (fileName.indexOf(`${k}.bin`) >= 0 && !entries[k]){
                  res.openReadStream(entry, function(err, readStream) {
                    if (err) return cb(err);
                    // ensure parent directory exists
                    var writebuffer = new streamBuffers.WritableStreamBuffer({});
                    readStream.pipe(writebuffer);
                    readStream.on('finish',() => {
                      console.log("readStream finish")
                      entries[k] = {
                        binary : writebuffer.getContents(),
                        version : (vers || index.latest).split("-v")[1].split(" ")[0] + "-release"
                      };
                      if (Object.keys(entries).every(j => entries[j] ? true : false)){
                        cb(null, entries)
                      }
                    })
                  });
                }
              })
            })
          })
        }));
    });
	})
}


module.exports = {
  get_index : get_index,
  get_dfu : get_dfu
};
